$(document).ready(function () {
    // ------------------------
    // Examples and help
    // ------------------------
    $("#toggle-help").click(function (e) {
        $("#help").toggle();
        $("#examples").toggle(false);
    });

    $("#toggle-examples").click(function (e) {
        $("#help").toggle(false);
        $("#examples").toggle();
    });

    // ------------------------
    // Editor Stuff
    // ------------------------

    var editor = CodeMirror.fromTextArea('code', {
        parserfile: ["parsepython.js"],
        stylesheet: "assets/css/codemirror/pythoncolors.css",
        path: "assets/js/codemirror/",
        lineNumbers: true,
        textWrapping: false,
        indentUnit: 4,
        height: "305px",
        fontSize: "10pt",
        autoMatchParens: true,
        parserConfig: {'pythonVersion': 2, 'strictErrors': true},
        initCallback: rest
    });


    function rest(editor) {
        editor.focus();
        editor.grabKeys(function (e) {
            if (e.keyCode === 13) {
                if (e.ctrlKey) {
                    run(false);
                } else if (e.shiftKey) {
                    run(true);
                }
            }
        }, function (e) {
            return (e.ctrlKey || e.shiftKey) && e.keyCode === 13;
        });
    }

    $("#run").click(function () {
        run(false);
    });

    function clear() {
        $('#edoutput').text('');
        $('#mycanvas').hide();
    }

    function builtinRead(x) {
        if (Sk.builtinFiles === undefined || Sk.builtinFiles["files"][x] === undefined)
            throw "File not found: '" + x + "'";
        return Sk.builtinFiles["files"][x];
    }

    // ------------------------
    // Skulpt
    // ------------------------
    function run(selection) {
        clear();
        var output = $('#edoutput');
        var outf = function (text) {
            output.text(output.text() + text);
        };

        Sk.configure({output: outf, read: builtinRead});

        Sk.canvas = "mycanvas";
        Sk.pre = "edoutput";

        try {
            eval(Sk.importMainWithBody("<stdin>", false, editor.getCode()));
            $('#console').show();
            $('#error').hide();
        } catch (e) {
            addErrorMessage(e);
            $('#error').show();
            $('#console').hide();
        }

        return false;
    }

    // ------------------------
    // Error messages
    // ------------------------
    var errorText = {};
    errorText.ParseError = "Ein 'parse error' meint, dass der Computer die Syntax an der angegeben Zeile nicht versteht. Tritt zum Beispiel auf, wenn ein Komma zwischen Argumenten oder ein Doppelpunkt nach einem if oder for Konstrukt fehlt.";
    errorText.ParseErrorFix = "Um einen 'parse error' zu beheben muss die angegebene Zeile genau untersucht werden. Findet sich dort keinen Fehler lohnt es sich die Zeile davor zu analysieren.";
    errorText.SyntaxError = "Ein 'parse error' meint, dass der Computer die Syntax an der angegeben Zeile nicht versteht. Tritt zum Beispiel auf, wenn ein Komma zwischen Argumenten oder ein Doppelpunkt nach einem if oder for Konstrukt fehlt.";
    errorText.SyntaxErrorFix = "Um einen 'parse error' zu beheben muss die angegebene Zeile genau untersucht werden. Findet sich dort keinen Fehler lohnt es sich die Zeile davor zu analysieren.";
    errorText.TypeError = "'Type errors' treten meistens auf, wenn versucht wird zwei Werte zu kombinieren, welche nicht kompatibel sind. Tritt zum Beispiel auf, wenn eine Zahl von einem String subtrahiert wird.";
    errorText.TypeErrorFix = "Um einen 'type error' zu beseitigen geht man durch den Code und überprüft, ob alle Variabeln einen sinnvollen Typ haben. Der Typ einer Variable kann wie folgt ausgegeben werden: 'print(type(my_variable))'";
    errorText.NameError = "Ein 'name error' tritt oft auf, wenn eine Variable benützt wird bevor ihr einen Wert zugewiesen wurde. Eventuell handelt es sich auch um einen Typo beim Variabelnamen.";
    errorText.NameErrorFix = "Um einen 'name error' zu beseitigen lohnt es sich alle Variabelnamen zu überprüfen. Werden die Variabeln vor dem Zugriff mit einem Wert initialisiert?";
    errorText.ValueError = "Ein 'value error' tritt meistens auf wenn einer Funktion ein Argument übergeben wird und diese einen anderen Typ erwartet.";
    errorText.ValueErrorFix = "Die Fehlermeldung gibt den Funktionsnamen an, wo der Fehler auftritt. Überprüfe den Typ der übergebenen Argumente.";
    errorText.AttributeError = "Einen 'attribute error' tritt auf, wenn das Objekt das verwendete Attribute oder die verwendete Methode nicht besitzt.";
    errorText.AttributeErrorFix = "Um einen 'attribute error' zu beseitigen muss überprüft werden ob das verwendete Attribute oder die verwendete Funktion im Objekt existiert.";
    errorText.TokenError = "Ein 'token error' tritt meistens auf wenn eine schliessende Klammer oder fehlt oder Anführungszeichen falsch gesetzt sind.";
    errorText.TokenErrorFix = "Um einen 'token error' zu beseiten muss überprüft werden, dass zu allen öffnenden Klammern eine schliessende existiert. Zu jedem Anführungszeichen muss ein zweites, schliessendes existieren.";
    errorText.IndexError = "Ein 'index error' tritt auf, wenn der Index in einem String oder in einer Liste nicht existiert. Zum Beispiel wenn eine Liste 3 Elemente beinhaltet und mit Index 3 zugegriffen wird.";
    errorText.IndexErrorFix = "In der Informatik beginnt ein Index mit 0. In einer Liste mit 3 Elementen hat das letzte Element den Index 2.";
    errorText.URIError = "";
    errorText.URIErrorFix = "";
    errorText.ImportError = "Ein 'import error' tritt auf wenn man ein Modul importieren möchte, welches nicht existiert.";
    errorText.ImportErrorFix = "Eventuell handelt es sich um einen Typo. Oder das Modul existiert in einer richtigen Python Umgebung, jedoch nicht in dieser reduzierten Browserumgebung.";
    errorText.ZeroDivisionError = "Divison durch 0 ist nicht erlaubt.";
    errorText.ZeroDivisionErrorFix = "Eine Überprüfung mit einem if Konstrukt, ob der Teiler nicht 0 kann Abhilfe schaffen.";
    errorText.InternalError = "Ein interner Fehler ist aufgetreten.";
    errorText.InternalErrorFix = "Bitte melden Sie diesen Fehler der Administration";
    errorText.IndentationError = "Ein 'indentation error' tritt auf wenn der Code nicht richtig eingerückt ist. Das passiert oft in der Nähe eines if, for, while oder def statement.";
    errorText.IndentationErrorFix = "Eine Überprüfung der if, for, while und def Konstrukte kann Abhilfe schaffen. Welche Zeilen gehören zum Block und welche nicht?";


    function addErrorMessage(err) {
        var errorString = err.toString();
        $('#error-string').html(errorString);

        var to = errorString.indexOf(":");
        var errorName = errorString.substring(0, to);
        var errorDesc = errorText[errorName];

        var errorDescFix = errorText[errorName + 'Fix'];

        $('#error-desc').html(errorDesc);
        $('#error-desc-fix').html(errorDescFix);
    }


    // ------------------------
    // Example code snippets
    // ------------------------

    var exampleCode = function (id, text) {
        $(id).click(function(e) {
            editor.setCode(text);
            editor.focus(); // so that F5 works, hmm
        });
    };
    exampleCode('#hello_world',
        "print('Hello World')"
    );
    exampleCode('#comments',
        "print('Hello World')  #Dies ist ein Kommentar\n" +
        "#Diese Zeile wird ignoriert"
    );
    exampleCode('#variables_types',
        "my_string = 'Hello World'  #String: Beliebige Zeichenkette, werden mit '' oder \"\" deklariert\n" +
        "my_int = 5  #Int: Ganze Zahl\n" +
        "my_float = 1.3333  #Float: Gleitkommazahl\n\n" +
        "#Gibt die Variabeln aus:\n" +
        "print(my_string)\n" +
        "print(my_int)\n" +
        "print(my_float)\n\n" +
        "#Schreibt eine leere Zeile\n"+
        "print\n\n"+
        "#Gibt die Typen der Variabeln aus:\n" +
        "print(type(my_string))\n" +
        "print(type(my_int))\n" +
        "print(type(my_float))\n"
    );
    exampleCode('#expressions',
        "#Variaben und Werte können kombiniert werden\n" +
        "print(2 + 2)\n" +
        "num1 = 2\n" +
        "print(num1 + 2)\n\n" +
        "#Strings können zusammengefügt werden\n" +
        "str1 = 'Hello'\n" +
        "str2 = 'World'\n" +
        "print(str1 + ' ' + str2)"
    );
    exampleCode('#builtin_functions',
        "#Aufruf von der print Funktion mit Parameter my_string\n" +
        "hello_world = 'Hello World'\n" +
        "print(hello_world)\n\n" +
        "#round rundet eine Kommazahl numerisch\n" +
        "a = 1.8888\n" +
        "a_rounded = round(a)\n" +
        "print(a_rounded)\n"
    );
    exampleCode('#math_functions',
        "#Import eines zusätzlichen Moduls\n" +
        "import math\n\n" +
        "#Beispiele:\n" +
        "print(math.log(17.0))\n" +
        "print(math.sin(45))\n" +
        "angle = 20\n" +
        "x = math.cos(angle + math.pi/2)\n" +
        "print(x)"
    );
    exampleCode('#boolean_operators',
        "int1 = 3\n" +
        "int2 = 10\n" +
        "print (int1 == int2)  #Gleich\n" +
        "print (int1 != int2)  #Ungleich\n" +
        "print (int1 > int2)   #Grösser\n" +
        "print (int1 >= int2)  #Grösser oder gleich\n" +
        "print (int1 < int2)   #Kleiner\n" +
        "print (int1 <= int2)  #Kleiner oder gleich\n"
    );
    exampleCode('#if_condition',
        "#Erstellt eine Zufallsvariable zwischen -10 und 10.\n" +
        "import random\n" +
        "random_int = random.randint(-10,10)\n" +
        "print 'random_int is: ', random_int\n\n" +
        "#Gibt aus ob die Zahl grösser, kleiner oder gleich 0 ist.\n"+
        "if random_int > 0:\n" +
        "    print('random_int is positive')\n" +
        "elif random_int == 0:\n" +
        "    print('random_int is zero')\n" +
        "else:\n" +
        "    print('random_int is negative')\n"
    );
});
